package com.bursie.harvester.logic;

import com.bursie.harvester.util.InternetQuery;
import com.google.gson.Gson;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

/**
 * A class for checking the relevance of a keyword compared with it's seed keyword.
 */
public class RelevanceChecker {
    private static final String GOOGLE_BASE_URL = "http://ajax.googleapis.com/ajax/services/search/web?v=1.0&q=";

    private Gson gson;
    private InternetQuery internetQuery;

    public RelevanceChecker() {
        this(new Gson(), new InternetQuery());
    }

    public RelevanceChecker(Gson gson, InternetQuery internetQuery) {
        this.gson = gson;
        this.internetQuery = internetQuery;
    }

    /**
     * Checks the relevance for each suggested keyword, according to number of pages indexed by Google where both
     * words are present
     *
     * @param seed The seed keyword
     * @param suggestedKeywords A Set of suggested keywords
     * @return A list of Word objects. The Word with the highest ranking is first in the list. A keyword which never
     *         co-occurs on a Google indexed page will get a ranking of 0. Words for which no ranking can be found, will get a
     *         ranking of 0 (the lowest possible ranking).
     */
    public List<Word> checkRelevance(String seed, Set<String> suggestedKeywords) {

        List<Word> rankedKeywords = new ArrayList<Word>(); // suggested keywords with ranking
        int coOccurringPages;  // number of pages which contains both seed word and a suggested keyword

        for (String suggestedKeyword : suggestedKeywords) {
            coOccurringPages = findNumberOfCoOccuringPages(seed, suggestedKeyword);
            rankedKeywords.add(new Word(suggestedKeyword, coOccurringPages));
        }
        Collections.sort(rankedKeywords);

        return rankedKeywords;
    }

    /**
     * Evaluates the relevance for a suggested keyword, by checking number of pages indexed by Google that contains both
     * words (seed and suggestedKeyword)
     *
     * @param seed The word for which we search related keywords
     * @param suggestedKeyword A keyword with unknown relevance to the seed
     * @return The number of pages indexed by Google that contains both words (seed and suggestedKeyword). Returns 0 if
     *         no pages with both words can be found, for example if there was an error
     */
    private int findNumberOfCoOccuringPages(String seed, String suggestedKeyword) {
        String query = "+\"" + seed + "\" +\"" + suggestedKeyword + "\"";   // force use of both words in the pages
        try {

            return findNumberOfGoogleResults(query);

        } catch (IOException e) {
            e.printStackTrace();
            return 0;
        }
    }

    /**
     * Performs a Google search and returns an estimate for the number of pages which contains the specific search string
     *
     * @param query A search string to send to Google
     * @return An estimate for the number of pages which contains the specific search string
     * @throws IOException in case something went wrong
     */
    private int findNumberOfGoogleResults(String query) throws IOException {
        // get a json formatted String using an internet query
        String json = internetQuery.getQueryResult(GOOGLE_BASE_URL, query);

        // read number of results from json using gson
        GsonGoogleObject gsonGoogleObject = gson.fromJson(json, GsonGoogleObject.class);
        String numberOfResults = gsonGoogleObject.getResponseData().getCursor().getEstimatedResultCount();

        if (numberOfResults == null) {
            return 0;
        }
        return Integer.parseInt(numberOfResults);
    }

    /**
     * The class for the main object returned by the Google request
     */
    private static class GsonGoogleObject {
        private GoogleResponseData responseData;
        private String responseDetails;
        private Integer responseStatus;

        public GoogleResponseData getResponseData() {
            return responseData;
        }
    }

    private static class GoogleResponseData {
        private List<GoogleResult> results;
        private GoogleCursor cursor;

        public GoogleCursor getCursor() {
            return cursor;
        }
    }

    private static class GoogleResult {
        private String GsearchResultClass;
        private String unescapedUrl;
        private String url;
        private String visibleUrl;
        private String cacheUrl;
        private String title;
        private String titleNoFormatting;
        private String content;
    }

    /**
     * This is the cursor class, that contains the estimated number of pages containing the search string
     */
    private static class GoogleCursor {
        private List<GooglePage> pages;
        private String estimatedResultCount;
        private Integer currentPageIndex;
        private String moreResultsUrl;

        public String getEstimatedResultCount() {
            return estimatedResultCount;
        }
    }

    private static class GooglePage {
        private String start;
        private Integer label;
    }

}
